@@ -47,6 +47,9 @@ type ('a, 'b) decoder = {
 type audio_info = { channels : int; sample_rate : int }
 type audio_data = float array array
 
+type audio_ba_data =
+  (float, Bigarray.float32_elt, Bigarray.c_layout) Bigarray.Array1.t array
+
 type video_plane =
   (int, Bigarray.int8_unsigned_elt, Bigarray.c_layout) Bigarray.Array1.t
 
@@ -82,6 +85,9 @@ type video_data = {
 type decoders =
   | Video of (video_info, video_data) decoder
   | Audio of (audio_info, audio_data) decoder
+  | Audio_ba of (audio_info, audio_ba_data) decoder
+  | Audio_both of
+      (audio_info, audio_data) decoder * (audio_info, audio_ba_data) decoder
   | Unknown
 
 type callbacks = {
@@ -170,9 +176,11 @@ let granuleconv dec granulepos cur =
   try
     let ret =
       match dec with
+        | Audio_ba d -> d.samples_of_granulepos granulepos
+        | Audio_both (d, _) -> d.samples_of_granulepos granulepos
         | Audio d -> d.samples_of_granulepos granulepos
         | Video d -> d.samples_of_granulepos granulepos
-        | _ -> assert false
+        | Unknown -> assert false
     in
     if ret > Int64.zero then ret else cur
   with _ -> cur
@@ -322,6 +330,10 @@ let fold_tracks dec f x =
 let get_track dec dtype =
   let test ended id stream =
     match (stream.dec, dtype) with
+      | Audio_ba _, Audio_track (_, x) when x = id ->
+          raise (Track (ended, id, stream))
+      | Audio_both _, Audio_track (_, x) when x = id ->
+          raise (Track (ended, id, stream))
       | Audio _, Audio_track (_, x) when x = id ->
           raise (Track (ended, id, stream))
       | Video _, Video_track (_, x) when x = id ->
@@ -339,6 +351,8 @@ let get_track dec dtype =
 let get_tracks dec =
   let f id stream l =
     match stream.dec with
+      | Audio_ba d -> Audio_track (d.name, id) :: l
+      | Audio_both (d, _) -> Audio_track (d.name, id) :: l
       | Audio d -> Audio_track (d.name, id) :: l
       | Video d -> Video_track (d.name, id) :: l
       | Unknown -> l
@@ -354,6 +368,8 @@ let drop_track dec dtype =
   (* Remove all track of this type *)
   let get_tracks id s l =
     match (s.dec, dtype) with
+      | Audio_ba _, Audio_track (_, x) when x = id -> (id, s) :: l
+      | Audio_both _, Audio_track (_, x) when x = id -> (id, s) :: l
       | Audio _, Audio_track (_, x) when x = id -> (id, s) :: l
       | Video _, Video_track (_, x) when x = id -> (id, s) :: l
       | _ -> l
@@ -378,6 +394,11 @@ let drop_track dec dtype =
 let get_standard_tracks ?tracks dec =
   let f id stream (a_t, v_t, l) =
     match stream.dec with
+      | Audio_ba d when a_t = None -> (Some (Audio_track (d.name, id)), v_t, l)
+      | Audio_ba d -> (a_t, v_t, Audio_track (d.name, id) :: l)
+      | Audio_both (d, _) when a_t = None ->
+          (Some (Audio_track (d.name, id)), v_t, l)
+      | Audio_both (d, _) -> (a_t, v_t, Audio_track (d.name, id) :: l)
       | Audio d when a_t = None -> (Some (Audio_track (d.name, id)), v_t, l)
       | Audio d -> (a_t, v_t, Audio_track (d.name, id) :: l)
       | Video d when v_t = None -> (a_t, Some (Video_track (d.name, id)), l)
@@ -399,6 +420,8 @@ let get_standard_tracks dec = get_standard_tracks dec
 let rec sample_rate_priv d dec =
   try
     match d with
+      | Audio_ba d -> ((fst (d.info ())).sample_rate, 1)
+      | Audio_both (d, _) -> ((fst (d.info ())).sample_rate, 1)
       | Audio d -> ((fst (d.info ())).sample_rate, 1)
       | Video d ->
           ((fst (d.info ())).fps_numerator, (fst (d.info ())).fps_denominator)
@@ -420,7 +443,7 @@ let get_position dec =
   then raise Not_available;
   let f _ stream pos =
     match stream.dec with
-      | Audio _ | Video _ -> min stream.position pos
+      | Audio_ba _ | Audio_both _ | Audio _ | Video _ -> min stream.position pos
       | _ -> pos
   in
   fold_tracks dec f max_float
@@ -533,7 +556,7 @@ let seek ?(relative = false) dec time =
       float x /. float y
     in
     match stream.dec with
-      | Audio _ ->
+      | Audio_ba _ | Audio_both _ | Audio _ ->
           {
             sync_id = id;
             sync_stream = stream;
@@ -584,6 +607,8 @@ let seek ?(relative = false) dec time =
   let resync x =
     sync_forward dec sync_points x;
     match x.sync_stream.dec with
+      | Audio_ba d -> d.restart x.sync_stream.os
+      | Audio_both (d, _) -> d.restart x.sync_stream.os
       | Audio d -> d.restart x.sync_stream.os
       | Video d -> d.restart x.sync_stream.os
       | _ -> ()
@@ -606,11 +631,20 @@ let incr_pos dec stream len =
 
 let rec audio_info dec dtype =
   let _, _, stream = get_track dec dtype in
-  try match stream.dec with Audio d -> d.info () | _ -> raise Not_found
+  try
+    match stream.dec with
+      | Audio_ba d -> d.info ()
+      | Audio_both (d, _) -> d.info ()
+      | Audio d -> d.info ()
+      | _ -> raise Not_found
   with Ogg.Not_enough_data ->
     feed dec;
     audio_info dec dtype
 
+let can_decode_ba dec dtype =
+  let _, _, stream = get_track dec dtype in
+  match stream.dec with Audio_ba _ | Audio_both _ -> true | _ -> false
+
 let rec video_info dec dtype =
   let _, _, stream = get_track dec dtype in
   try match stream.dec with Video d -> d.info () | _ -> raise Not_found
@@ -618,16 +652,16 @@ let rec video_info dec dtype =
     feed dec;
     video_info dec dtype
 
-let decode_audio dec dtype f =
+let decode_audio_gen ~get_decoder ~length dec dtype f =
   let ended, id, stream = get_track dec dtype in
   try
     let f x =
       begin
-        try incr_pos dec stream (Array.length x.(0)) with _ -> ()
+        try incr_pos dec stream (length x.(0)) with _ -> ()
       end;
       f x
     in
-    match stream.dec with Audio d -> d.decode f | _ -> assert false
+    (get_decoder stream.dec).decode f
   with
   | ( End_of_stream
     (* In very rare cases (e.g. with a track that
@@ -646,6 +680,24 @@ let decode_audio dec dtype f =
     else if e = Ogg.Not_enough_data then raise e;
     if eos dec then raise End_of_stream
 
+let decode_audio =
+  let get_decoder = function
+    | Audio d -> d
+    | Audio_both (d, _) -> d
+    | _ -> raise Not_available
+  in
+  let length = Array.length in
+  decode_audio_gen ~get_decoder ~length
+
+let decode_audio_ba =
+  let get_decoder = function
+    | Audio_ba d -> d
+    | Audio_both (_, d) -> d
+    | _ -> raise Not_available
+  in
+  let length = Bigarray.Array1.dim in
+  decode_audio_gen ~get_decoder ~length
+
 let decode_video dec dtype f =
   let ended, id, stream = get_track dec dtype in
   try
@@ -674,4 +726,5 @@ let decode_rec g dec dtype f =
   exec ()
 
 let decode_audio = decode_rec decode_audio
+let decode_audio_ba = decode_rec decode_audio_ba
 let decode_video = decode_rec decode_video
